local TimerFu = AceLibrary("AceAddon-2.0"):new("FuBarPlugin-2.0", "AceDB-2.0", "AceConsole-2.0", "Metrognome-2.0")
TimerFu:RegisterDB("TimerFuDB")
TimerFu:RegisterDefaults('profile', {
	counting = false,
	message = true,
	sound = true,
	loop = true,
})

TimerFu.hasIcon = "Interface\\Icons\\INV_Misc_PocketWatch_01"
TimerFu.defaultPosition = "CENTER"

local L = AceLibrary("AceLocale-2.2"):new("TimerFu")

local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local crayon = AceLibrary("Crayon-2.0")

local function getTimerString(value)
	local s, e, hours, minutes, seconds
	s, e, minutes = string.find(value, "^(%d+)$")
	if s then
		return tonumber(minutes)*60
	end
	s, e, minutes, seconds = string.find(value, "^(%d+):(%d+)$")
	if s then
		return tonumber(minutes)*60+tonumber(seconds)
	end
	s, e, hours, minutes, seconds = string.find(value, "^(%d+):(%d+):(%d+)$")
	if s then
		return tonumber(hours)*3600+tonumber(minutes)*60+tonumber(seconds)
	end
end

function TimerFu:OnInitialize()
	local options = {
		desc = L["An addon to track time"],
		type = "group",
		args = {
			start = {
				name = L["start"],
				desc = L["Start counting"],
				type = "execute",
				func = function () 
					if not self.db.profile.counting then
						self.db.profile.counting = true
						self.db.profile.decounting = false
						self.db.profile.target = time()
					end
				end,
			},
			toggle = {
				name = L["toggle"],
				desc = L["Toggle pause"],
				type = "execute",
				func = function ()
					if self.db.profile.counting then
						self:TogglePause()
					end
				end,
			},
			pause = {
				name = L["pause"],
				desc = L["Pause counting"],
				type = "execute",
				func = function ()
					if self.db.profile.counting and not self.db.profile.paused then
						self:TogglePause()
					end
				end,
			},
			unpause = {
				name = L["unpause"],
				desc = L["Restart counting if paused"],
				type = "execute",
				func = function ()
					if self.db.profile.counting and self.db.profile.paused then
						self:TogglePause()
					end
				end,
			},
			reset = {
				name = L["reset"],
				desc = L["Reset the timer"],
				type = "execute",
				func = function ()
					self:OnDoubleClick()
				end,
			},
			
			set = {
				name = L["set"],
				desc = L["Set the counter for a given time"],
				usage= L["set <time>"],
				type = "text",
				get = false,
				set = function (v) self:StartTimerString(v) end,
				validate = getTimerString,
			},
			message = {
				name = L["message"],
				desc = L["Show a message in ChatFrame when the timer reaches 0"],
				type = "toggle",
				get = function () return self.db.profile.message end,
				set = function (v) self.db.profile.message = v end,
			},
			sound = {
				name = L["sound"],
				desc = L["Play a sound when the timer reaches 0"],
				type = "toggle",
				get = function () return self.db.profile.sound end,
				set = function (v) self.db.profile.sound = v end,
			},
			loop = {
				name = L["loop"],
				desc = L["Loop the counter when the timer reaches 0"],
				type = "toggle",
				get = function () return self.db.profile.loop end,
				set = function (v) self.db.profile.loop = v end,
			},
		}
	}
	self:RegisterChatCommand(L["CHAT_COMMANDS"], options)
end

function TimerFu:OnEnable()
	self:RegisterMetro(self.name, self.UpdateText, 1, self)
	self:StartMetro(self.name)
end

function TimerFu:FormatTime(s)
	local t
	if s < 0 then
		t = "-"
		s = -s
	else
		t = ""
	end
	local h, m
	h = math.floor(s / 3600); s = s - (h * 3600)
	m = math.floor(s / 60); s = s - (m * 60)
	return string.format("%s%2.2d:%2.2d:%2.2d", t, h, m, s)
end

function TimerFu:OnTextUpdate()
	local p = self.db.profile
	local t, color
	if not p.counting then
		t = 0
		color = crayon.White
	else
		if p.paused then
			color = crayon.Orange
			t = - p.remaining
		else
			color = crayon.Green
			t = time() - p.target
			if p.decounting and math.floor(t) == 0 then
				if p.message then
					self:Print(L["Timer has finished !"])
				end
				if p.sound then
					PlaySound("TellMessage")
				end
				if p.loop then
					p.target = time() + p.last_value
				else
					p.counting = false
				end
			end
		end
	end
	self:SetText(color(crayon, self:FormatTime(t)))
end

function TimerFu:OnTooltipUpdate()
	-- cat = table:AddCategory()
	tablet:SetHint(L["Tip Hint"])
end

function TimerFu:StartTimer(count)
	self.db.profile.counting = true
	self.db.profile.decounting = true
	if self.db.profile.paused then
		self.db.profile.remaining = count
	else
		self.db.profile.target = time() + count
	end
	self:UpdateText()
end

function TimerFu:StartTimerString(value)
	value = getTimerString(value)
	if value then
		self.db.profile.counting = true
		self.db.profile.decounting = true
		self.db.profile.last_value = value
		if self.db.profile.paused then
			self.db.profile.remaining = value
		else
			self.db.profile.target = time() + value
		end
		self:UpdateText()
	end
end

function TimerFu:OnMenuRequest(level, value)
	if level == 1 then
		dewdrop:AddLine(
			'text', L["message"],
			'checked', self.db.profile.message,
			'func', function () self.db.profile.message = not self.db.profile.message end,
			'tooltipTitle', L["message"],
			'tooltipText', L["Show a message in ChatFrame when the timer reaches 0"]
			)
		dewdrop:AddLine(
			'text', L["sound"],
			'checked', self.db.profile.sound,
			'func', function () self.db.profile.sound = not self.db.profile.sound end,
			'tooltipTitle', L["sound"],
			'tooltipText', L["Play a sound when the timer reaches 0"]
			)
		dewdrop:AddLine(
			'text', L["loop"],
			'checked', self.db.profile.loop,
			'func', function () self.db.profile.loop = not self.db.profile.loop end,
			'tooltipTitle', L["loop"],
			'tooltipText', L["Loop the counter when the timer reaches 0"]
			)
		dewdrop:AddLine(
			'closeWhenClicked', true,
			'text', L["Pause"],
			'func', function() self:TogglePause() end,
			'checked', self.db.profile.paused
			)
		dewdrop:AddLine(
			'closeWhenClicked', true,
			'text', L["Reset"],
			'func', function() 
						self.db.profile.counting = false 
						self.db.profile.paused = false 
					end
			)
		dewdrop:AddLine(
			'closeWhenClicked', true,
			'text', L["Start counting"],
			'func', function() 
						self.db.profile.counting = true 
						self.db.profile.decounting = true 
						self.db.profile.target = time()
						self.db.profile.paused = false 
					end,
			'checked', self.db.profile.paused
			)
		dewdrop:AddLine(
			'text', L["Start a Timer"],
			'hasArrow', true,
			'value', "timing"
		)
	elseif level == 2 then
		if value == "timing" then
			dewdrop:AddLine(
				'closeWhenClicked', true,
				'text', string.format(L["1 minute"], n),
				'func', self.StartTimer,
				'arg1', self,
				'arg2', 60
			)
			for _, n in ipairs({2, 5, 10, 15, 30, 60}) do
				dewdrop:AddLine(
					'closeWhenClicked', true,
					'text', string.format(L["%d minutes"], n),
					'func', self.StartTimer,
					'arg1', self,
					'arg2', n*60
				)
			end
			dewdrop:AddLine(
				'closeWhenClicked', true,
				'text', string.format(L["Other timing"], n),
				'hasArrow', true,
				'value', "timingOther",
				'hasEditBox', true,
				'editBoxText ', "00:00:00",
				'editBoxFunc',self.StartTimerString,
				'editBoxArg1',self,
				'editBoxValidateFunc ', getTimerString
			)
		end
	end
end

function TimerFu:TogglePause()
	if self.db.profile.paused then
		self.db.profile.paused = false
		self.db.profile.target = time() + self.db.profile.remaining
	else
		self.db.profile.paused = true
		self.db.profile.remaining = self.db.profile.target - time()
	end
end

function TimerFu:OnDoubleClick()
	self.db.profile.counting = false
	self.db.profile.paused = false
	self:UpdateText()
end

function TimerFu:OnClick()
	if self.db.profile.paused or self.db.profile.counting then
		self:TogglePause()
	else
		self.db.profile.counting = true
		self.db.profile.decounting = false
		self.db.profile.target = time()
	end
	self:UpdateText()
end

